/*=========================================================================
Program: UVT
Autor: Pavel TATAR
Hardware: Pavel TATAR
Datum: 07 - 10/2002
Popis: Udrzovani vhlkosti a teploty s prevodem hodnot po RS232/485
Dokumentace: UVT.xls - Komunikacni protokol.
==========================================================================*/
#include <at89x52.h>
//#include <AT898252.H>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <intrins.h>
#include <stdlib.h>


#define	AT8252		0		// volba typu procesoru 0 = 89c52, 1 = 89S8252

#define I2C_M_ADDR 	0xA0
#define	RD			1
#define	WR			0


sbit	WDT =	P1 ^ 0;		// WatchDog Timer
sbit	HEAT =	P1 ^ 1;		// topeni
sbit	VENT =	P1 ^ 2;		// ventilace
sbit	WATER =	P1 ^ 3;		// zalevani

sbit	DA0	 	=	P2 ^ 0;		// display 1
sbit	DA1	 	=	P2 ^ 1;		// display 2
sbit	DA2		=	P2 ^ 2;		// display 3
sbit	DA3		=	P2 ^ 3;		// display 4
sbit	DALED	=	P2 ^ 4;		// ledky
sbit	SCL	 	=	P2 ^ 6;		// i2c hodiny
sbit	SDA		=	P2 ^ 7;		// i2c data


sbit	HIN	 =	P3 ^ 2;		// vstup vlhkost
sbit	TIN	 =	P3 ^ 3;		// vstup teplota
sbit	DIR	 =	P3 ^ 7;		// smer komunikace


sfr CATHODE_PORT = 0x80;		// port 0

bit		mest 		= 0;	// priznak mereni casu
bit		mesfi 		= 0;	// priznak mereni prvniho impulzu
bit		messtart 	= 0;	// priznak startu mereni
bit		mesdone 	= 0;	// priznak uspesneho mereni
bit		transmit	= 0;	// priznak vysilani
bit		receive		= 0;	// priznak prijmu
bit		receive_p	= 0;	// priznak prijmu prefixu
bit		received	= 0;	// priznak konce prijmu
bit		rectout		= 0;	// priznak timeoutu prijmu
bit		eor			= 0;	// priznak zpravy o timeoutu prijmu
bit		send_n		= 0;	// priznak posilani adresy
bit		s_hum 		= 0;	// povoleni posilani vlkosti po kazdem odmeru
bit		s_temp 		= 0;	// povoleni posilani teploty po kazdem odmeru
bit		res_timer	= 0;	// priznak resetovaci prodlevy
bit		wdt_t2_en	= 0;	// povoleni resetovani WDT
bit		n_sec		= 0;	// priznak nove sekundy
bit		d_point		= 0;	// priznak des. tecky
bit		uti_error	= 0;	// priznak chyby uti
bit		smt_error	= 0;	// priznak chyby cidla teploty SMT
bit		temp_minus	= 0;	// priznak zaporne teploty
bit		h_v_en		= 0;	// povoleni funkce ventilace pri zvysene vhkosti
bit		t_v_en		= 0;	// povoleni funkce ventilace pri zvysene teplote
bit		lt_w_en 	= 0;	// povoleni funcke nezalevani pri nizke teplote 
bit		loc_heat_en	= 0;	// zapnuti topeni podle teploty
bit		loc_v_t_en	= 0;	// zapnuti ventilatoru podle teploty
bit		loc_v_h_en	= 0;	// zapnuti ventilatoru podle vlhkosti
bit		loc_wat_en	= 0;	// zapnuti zalevani podle teploty
bit		com_heat_en	= 0;	// zapnuti topeni komunikaci
bit		com_vent_en	= 0;	// zapnuti vetrani komunikaci
bit		com_wat_en	= 0;	// zapnuti zalevani komunikaci
bit		man_heat	= 0;	// manualni ovladani topeni
bit		man_vent	= 0;	// manualni ovladani vetrani
bit		man_wat		= 0;	// manualni ovladani zalevani
bit		cal_a_ok	= 0;	// priznak zkalibrovani a
bit		calib_ok	= 0;	// priznak zakalibrovani



unsigned int	phase[3];			// faze mereni
unsigned int	mtimep;				// delka mereneho pulzu teploty 
unsigned int	mtimedc;			// merena strida teploty 
int				temperature;		// celociselna hodnota teploty
unsigned char	temperature_dec;	// desetina teploty
unsigned char	humidity;			// vlhkost


unsigned char	s_count = 0;		// citac vyslanych byte
unsigned char	s_nob;				// pocet byte k vyslani

unsigned char 	*p_s_out;			// pointer na vysilana data

unsigned char command[15];			// prijaty comand
unsigned char message[10];			// vysilana zprava

unsigned char display_buffer[] = {0x0a, 0x0a, 0x0a, 0x0a};
unsigned char bdata led_buffer = 0xff;

sbit led_ce 	= led_buffer  ^ 0;	// LED komunikace/error
sbit led_heat 	= led_buffer  ^ 1;	// LED topeni
sbit led_vent 	= led_buffer  ^ 2;	// LED ventilace
sbit led_water 	= led_buffer  ^ 3;	// LED zalevani

unsigned char bdata led_flash = 0x00;

sbit led_lf_ce 		= led_flash  ^ 0;	// dlouhy blik LED komunikace/error
sbit led_f_heat 	= led_flash  ^ 1;	// blikani LED topeni
sbit led_f_vent 	= led_flash  ^ 2;	// blikani LED ventilace
sbit led_f_water	= led_flash  ^ 3;	// blikani LED zalevani
sbit led_sf_ce 		= led_flash  ^ 4;	// kratky blik LED komunikace/error
sbit led_sf_heat 	= led_flash  ^ 5;	// kratky blik LED topeni
sbit led_sf_vent 	= led_flash  ^ 6;	// kratky blik LED ventilace
sbit led_sf_water	= led_flash  ^ 7;	// kratky blik LED zalevani

#if AT8252							// pro AT8252 jsou nastavene hodnoty
									// ve vnitrni EEPROM a pri startu se kopiruji
unsigned char xdata *p_eeprom;		// ukazatel na data v EEPROM

unsigned char xdata ee_addr _at_ 0x00;		// eeprom nastavena adresa
unsigned char xdata ee_set_temp _at_ 0x01;	// eeprom nastavena teplota
unsigned char xdata ee_set_t_hyst _at_ 0x02;// eeprom nastavena teplotni hystereze
unsigned char xdata ee_set_hum _at_ 0x03;	// eeprom nastavena vhkost
unsigned char xdata ee_set_h_v _at_ 0x04;	// eeprom nastavena vhkost pro ventilaci
unsigned char xdata ee_set_t_v _at_ 0x05;	// eeprom nastavena teplota pro ventilaci
unsigned char xdata ee_set_lt_w _at_ 0x06;	// eeprom nastavena teplota pro blokovan zalevani
unsigned int xdata	ee_calib_k_a _at_ 0x07;	// eeprom kalibracni konstanta a
#define				EE_CALIB_K_A	7		// eeprom kalibracni konstanta a
unsigned char xdata	ee_calib_k_b _at_ 0x09;	// eeprom kalibracni konstanta b
unsigned char xdata	ee_calib_k_ok _at_ 0x0a;	// eeprom priznak zakalibrovani

#else

unsigned char p_eeprom;				// ukazatel na data v i2c EEPROM, zde to neni pointer

#define 	EE_ADDR	  		0		// eeprom nastavena adresa
#define		EE_SET_TEMP		1		// eeprom nastavena teplota
#define		EE_SET_T_HYST 	2		// eeprom nastavena teplotni hystereze
#define		EE_SET_HUM		3		// eeprom nastavena vhkost
#define		EE_SET_H_V 		4		// eeprom nastavena vhkost pro ventilaci
#define		EE_SET_T_V 		5		// eeprom nastavena teplota pro ventilaci
#define		EE_SET_LT_W		6		// eeprom nastavena teplota pro blokovan zalevani
#define		EE_CALIB_K_A	7		// eeprom kalibracni konstanta a
#define		EE_CALIB_K_B	9		// eeprom kalibracni konstanta b
#define		EE_CALIB_K_OK	10		// eeprom priznak zakalibrovani

#endif
									// pro 89C52 jsou nastavene hodnoty
									// ve vnejsi EEPROM a pri startu se kopiruji
unsigned char idata addr;			// nastavena adresa
unsigned char idata set_temp;		// nastavena teplota
unsigned char idata set_t_hyst;		// nastavena teplotni hystereze
unsigned char idata set_hum;		// nastavena vhkost
unsigned char idata set_h_v;		// nastavena vhkost pro ventilaci
unsigned char idata set_t_v;		// nastavena teplota pro ventilaci
unsigned char idata set_lt_w;		// nastavena teplota pro blokovan zalevani
unsigned int  idata	calib_k_a;		// kalibracni konstanta a
unsigned char idata	calib_k_b;		// kalibracni konstanta b
unsigned char idata	calib_k_ok;		// priznak zakalibrovani




char code char_tab[] = {								// tabulka znaku INVERTOVANA !!!
                        0x3f, 0x06, 0x5b, 0x4f, 0x66,	// 0-4
                        0x6d, 0x7d, 0x07, 0x7f, 0x6f,	// 5-9
                        0x00,							// 0a = nic
						0xf6, 0x79, 0x50, 0x40			// 0b = H, E, r, -
                       };



//-------------------------------definice textu

unsigned char code name[] = "UTV1 v1.1";
unsigned char code quickly[] = "EoR!\r";
unsigned char code error[] = "Error\r";
unsigned char code merror[] = "Manual!\r";
unsigned char code terror[] = "T Error\r";
unsigned char code herror[] = "H Error\r";
unsigned char code calaf[] = "Calib a!\r";

unsigned char code ok[] = "OK\r";



/*##################################################################

						Vypocenti funkce

##################################################################*/

/*========================================================
Funkce vypoctu namerenych hodnot z obvodu UTI03

Vstup: 3byte kde plati
        1. moff = cas offsetu
        2. mx   = cas merene veliciny
	    3. mref = cas referencni veliciny
Vystup: vysledek
========================================================*/
unsigned int uti_calc(unsigned int moff, unsigned int mx, unsigned int mref)
 {
  unsigned long a = 0, b = 0;	// pomocne pro vypocet	

  if (moff >= mx)				// napred kontola zadanych hodnot
   return (0);
  if (mref <= mx)
   return (0xffff);
  a = (mx - moff);				// pote vypocet
  a = a << 16;
  b = mref - moff;
  return(a/b);					// navrat s vysledkem
 }

/*========================================================
vypocet teploty z obvodu SMT 160

Prepocita hodnoty casu na teplotu
========================================================*/
void temp_calc(void)
 {
  unsigned long a;

  a = (((long)mtimedc * 10) / 100);	// a = 1%
  a = (((long)mtimep * 100) / a);	// strida v procentech
  a *= 1000;
  a = ((a - 320000) / 470);			// vypocet teploty 
  temperature = a / 10;				// vypocet teploty
  temperature_dec = a % 10;			// vypocet desetin
  if (temperature < 0)				// test zaporne teploty
   {
    temp_minus = 1;					// pouze oznacit
    temperature = 0;				// nulova teplota
    temperature_dec = 1;			// a 1 desetina
   }
  else
   temp_minus = 0;

 }

/*##################################################################

					Funkce obsluhy hardware

##################################################################*/

#if AT8252
#else

/*========================================================
Obsluha I2C
========================================================*/

/*---------------------------------------------------------
Cekaci smycka
----------------------------------------------------------*/
void I2C_Wait(void)
 {
  _nop_();
  _nop_();
  _nop_();
 }

/*---------------------------------------------------------
I2C Start
----------------------------------------------------------*/
void I2C_Start(void)
 {
  SCL = 1;
  I2C_Wait();
  SDA = 0;
  I2C_Wait();
  SCL = 0;
  I2C_Wait();
 }

/*---------------------------------------------------------
I2C Stop
----------------------------------------------------------*/
void I2C_Stop(void)
 {
  SDA = 0;
  I2C_Wait();
  SCL = 1;
  I2C_Wait();
  SDA = 1;
  I2C_Wait();
 }

/*---------------------------------------------------------
I2C cteni byte

vstup:	bit ack kde 1 = posledni byte
vystup:	precteny byte
----------------------------------------------------------*/
unsigned char I2C_GetByte(bit ack)
 {
  register unsigned char i, byte = 0;

  for (i = 0; i < 8; i++)	// cist 8 bitu, MSB prvni
   {
    SCL = 1;				// nahodit hodiny
	I2C_Wait();				// pockat na platna data
	byte <<= 1;				// rotovat byte
    if (SDA == 1)			// zapsat precteny bit
     byte++;
    SCL = 0;				// shodit hodiny
	I2C_Wait();
   }
  SDA = ack;             	// nastaveni ACK
  SCL = 1;					// hodinovy pulz
  I2C_Wait();
  SCL = 0;
  SDA = 1;					// uvolnit SDA
  return(byte);				// navrat s vysledkem
 }

/*---------------------------------------------------------
I2C zapis byte

vstup:	zapisovany byte
vystup:	ACK err/Ok
----------------------------------------------------------*/
bit I2C_SendByte(unsigned char byte)
 {
  register unsigned char i;

  for (i=0; i<8; i++)			// posilani 8 bitu, MSB prvni
   {
    ((byte & 0x80) == 0x80) ? (SDA = 1) : (SDA = 0); // zapis bitu na SDA
    I2C_Wait();					// pro jistotu pockat na ustaleni dat
    SCL = 1;					// hodinovy pulz
    I2C_Wait();
    SCL = 0;
    I2C_Wait();
	byte <<= 1;					// rotovat byte
  }
  SDA = 1;               		// uvolnit SDA
  SCL = 1;
  I2C_Wait();
  if (SDA == 1)					// test ACK
   {
    SCL = 0;
    return(0);					// pokud ne tak navrat s chybou
   }
  SCL = 0;
  return(1);
 }

/*---------------------------------------------------------
I2C zapis i2c adresy 

vstup:	zapisovany adresa a R/W
----------------------------------------------------------*/
void I2C_SendAddr(unsigned char addr, bit rw)
 {
  I2C_Start();									// i2c start
  (rw == 1) ? (addr |= 0x01) : (addr &= 0xfe);	// addr a R/W
  I2C_SendByte(addr);							// posilani i2c addresy
 }

/*---------------------------------------------------------
I2C cteni 1 byte na zadane adrese

vstup:	adresa byte
vystup:	precteny byte
----------------------------------------------------------*/
unsigned char I2C_RRead_Byte(unsigned char addr)
 {
  register unsigned char byte = 0;

  I2C_SendAddr(I2C_M_ADDR, WR);	// i2c adresa
  I2C_SendByte(addr);				// adresa bytu
  I2C_SendAddr(I2C_M_ADDR, RD);	// i2c adresa + cteni
  byte = I2C_GetByte(1);			// precist byte
  I2C_Stop();						// i2c stop
  return(byte);						// navrat s vysledkem
 }

#endif

/*========================================================
Zapis byte do EEPROM
========================================================*/
void ee_write (unsigned char byte)
 {

#if AT8252

  WMCON |= EEMEN_ | EEMWE_;			// povolit zapis
  *p_eeprom = byte;					// zapsat byte
  p_eeprom++;						// a zvyseni ukazatele
  while((WMCON & EERDY_) == 0);		// testovat dokonceni zapisu
  WMCON &= ~(EEMEN_ | EEMWE_);		// zakazat zapis do eeprom

#else

  I2C_SendAddr(I2C_M_ADDR, WR);
  I2C_SendByte(p_eeprom);
  I2C_SendByte(byte);
  I2C_Stop();
  p_eeprom++;
    
#endif
  }

/*========================================================
Test konce vysilani s timeoutem

Otestuje zda-li bylo ukonceno vysilani jinak ceka na konec,
pokud nedojde k ukonceni do 500ms povazuje vysilani za
ukoncene 
========================================================*/
void waitsend(void)
 {
  register unsigned int a = 0;		// pro timeout cekani na konec predchoziho

  while (transmit == 1) 			// napred pockat na konec vysilani predchoziho
   {
    a++;							// pro jistotu obsahuje timeout
	_nop_();
	_nop_();
	_nop_();
	_nop_();
	_nop_();
	if (a == 0xffff)				// cca 500mS
	 break;
   }
  transmit = 0;
 }
/*========================================================
Zahajeni vysilani

vysle prvni znak, dalsi uz budou vyslany v preruseni
vstup: pointer na retezec
========================================================*/
void send(unsigned char *p_x)
 {
  register unsigned int a = 0;		// pro timeout cekani na konec predchoziho
  register unsigned char b;

  if(transmit == 1)
   waitsend();
  DIR = 1;							// zapnout vysilani v rezimu 485
  p_s_out = p_x;
  s_nob = strlen(p_s_out);			// zjistit delku retezce
  b = *p_s_out;	    				// prvni znak ulozit
  p_s_out++;						// increment pointeru
  s_count = 1;						// init citace byte
  transmit = 1;						// nastaveni priznaku vysilani
  led_ce = 0;						// rozsvitit LED
  SBUF = b;							// vyslat
 }


/*========================================================
Nulovani WDT
========================================================*/
void res_wdt(void)
 {
#if AT8252
  WMCON |= WDTRST_;
#endif
  WDT = 0;
  _nop_();
  _nop_();
  _nop_();
  WDT = 1;
  
 }

/*========================================================
Mereni UTI 03 (vhkosti)
========================================================*/

void mes_uti(void)
 {
  unsigned char btcon, btmod, bth0;	// zaloha nastaveni citacu

  EA = 0;							// zakaz preruseni
  btcon = TCON;						// zaloha nastaveni citacu
  btmod = TMOD;
  bth0 = TH0;
  TCON = 0x00;
  TMOD = 0x01;						// tim0 16bit, tim1 16bit gate
  TH0 = 0x00;						// nulovat citace
  TL0 = 0x00;
  TR0 = 1;
  res_wdt();
  messtart = 1;
  uti_error = 0;
  ES = 0;							// po dobu mereni zakazat preruseni serialu
  EX0 = 1;							// povolit preruseni merenym signalem
  IT0 = 1;							// sestupnou hranou
  EA = 1;
  while(mesdone != 1)				// cekani na dokonceni mereni
   {
    if (uti_error == 1)
     break;
   }
  EA = 0;
  EX0 = 0;							// zakazat preruseni merenym signalem
  ES = 1;
  TH0 = bth0;						// obnovit nastaveni citacu
  TMOD = btmod;
  TCON = btcon;
  mesdone = 0;
  TH0 = 0xf9;						// nastavit timer0 zpet
  EA = 1;							// obnovit preruseni
  return;
 }

/*========================================================
Mereni teploty cidlem SMT160
========================================================*/

bit mes_smt1(void)					// funkce samotneho mereni
 {
  register unsigned int a = 0;
  register unsigned char b = 0;

  while(TIN == 0)					// cekani na pulz
   {
    a++;
	if(a == 10000)					// timeout
	 return 1;
   }
  a = 0;
  while(TIN == 1)					// cekani na konec pulzu
   {
    a++;
	if(a == 10000)					// timeout
	 return 1;
   }								
  TR1 = 1;							// v mezere povolit tim1
  while(TIN == 0);					// pockat na pulz
  TR0 = 1;							// spustit tim0
  for(b = 0;  b<= 24; b++)			// merit se bude 24 period
   {
    while(TIN == 1);				// pockat na mezeru
    while(TIN == 0);				// pockat na pulz
   }								// 25 perioda uz mimo cyklus
  while(TIN == 1);					// pockat na mezeru
  while(TIN == 0);					// pockat na pulz
  TR1 = 0;							// poradi vypinani citacu kompenzuje
  TR0 = 0;							// chybu pri zapinani
  mtimep = 0;						// pro jistotu
  mtimep = TH1 << 8;				// timer ulozeny jako byte
  mtimep +=TL1;						// prepocitat na int
  mtimedc = 0;						// pro jistotu
  mtimedc = TH0 << 8;				// timer ulozeny jako byte
  mtimedc +=TL0;					// prepocitat na int
  return 0;
 }

//=================================== nastaveni a spusteni mereni
void mes_smt(void)
 {
  unsigned char btcon, btmod, bth1;	// zaloha nastaveni citacu

  EA = 0;							// zakaz preruseni
  btcon = TCON;						// zaloha nastaveni citacu
  btmod = TMOD;
  bth1 = TH1;
  TCON = 0x00;
  TMOD = 0x91;						// tim0 16bit, tim1 16bit gate
  TH0 = 0x00;						// nulovat citace
  TL0 = 0x00;
  TH1 = 0x00;
  TL1 = 0x00;
  smt_error = mes_smt1();			// volani samotneho mereni
  TH1 = bth1;						// obnovit nastaveni citacu
  TMOD = btmod;
  TCON = btcon;
  EA = 1;							// obnovit preruseni
  return;							// navrat OK
 }

/*========================================================
Vysilani vhkosti
========================================================*/
void send_hum(unsigned char m)
 {

  sprintf (message, "*M%cH%c", addr, m);
  send(&message);
  waitsend();

  if (m == 'A')
   {
    sprintf (message, "%u%%\r", (unsigned int)humidity);
    send(&message);
    waitsend();
   }
  else
   {
    sprintf (message, "%u.%u\r", (unsigned int)set_hum, (unsigned int)set_h_v);
    send(&message);
    waitsend();
   }

 }


/*========================================================
Vysilani teploty
========================================================*/
void send_temp(unsigned char m)
 {
  sprintf (message, "*M%cT%c", addr, m);
  send(&message);
  waitsend();
  if (m == 'A')
   {
    sprintf (message, "%d.%u C\r", temperature, (unsigned int)temperature_dec);
    send(&message);
    waitsend();
   }
  else
   {
    sprintf (message, "%u.%u.", (unsigned int)set_temp, (unsigned int)set_t_hyst);
    send(&message);
    waitsend();
    sprintf (message, "%u.%u\r", (unsigned int)set_t_v, (unsigned int)set_lt_w);
    send(&message);
    waitsend();
   }
 }

/*========================================================
Vysilani kalibracnich konstant
========================================================*/
void read_calib(void)
 {
  unsigned char a;

  sprintf (message, "*M%cCC", addr);
  send(&message);
  waitsend();
  sprintf (message, "%u.%u.", calib_k_a, (unsigned int)calib_k_b);
  send(&message);
  (calib_ok == 1) ? (a = 'T') : (a = 'F');
  waitsend();
  sprintf (message, "%c\r", a);
  send(&message);
  waitsend();
 }

/*========================================================
Vysilani stavu vystupu
========================================================*/
void send_out_s(bit a, unsigned char b, bit c)
 {
  unsigned char m;
  unsigned char ts[4];

  (c == 1) ? (m = 'M') : (m = 'A');
  (a == 1) ? (sprintf(ts, "OFF")) : (sprintf(ts, "ON"));
  sprintf (message, "*M%c%c:%c.%s\r", addr, b, m, ts);
  send(&message);
  waitsend();
 }

/*========================================================
Vysilani hlaseni OK
========================================================*/
void send_ok(void)
 {
  sprintf (message, "*M%c%s\r", addr, ok);
  send(&message);
  waitsend();
 }

/*========================================================
Vysilani hlaseni Error
========================================================*/
void send_error(void)
 {
  sprintf (message, "*M%c%s\r", addr, error);
  send(&message);
  waitsend();
 }
/*========================================================
Vysilani hlaseni End of Recieve
========================================================*/
void send_quickly(void)
 {
  sprintf (message, "*M%c", addr);
  send(&message);
  waitsend();
  send(&quickly);
  waitsend();
 }

/*========================================================
Vysilani hlaseni Manual first
========================================================*/
void send_merror(void)
 {
  sprintf (message, "*M%c", addr);
  send(&message);
  waitsend();
  send(&merror);
  waitsend();
 }

/*========================================================
Vysilani hlaseni Calib a first
========================================================*/
void send_calaf(void)
 {
  sprintf (message, "*M%c", addr);
  send(&message);
  waitsend();
  send(&calaf);
  waitsend();
 }


/*========================================================
Vysilani jmena a adresy 
========================================================*/
void send_name(void)
 {
  sprintf (message, "*M%c ", addr);
  send(&message);
  waitsend();
  sprintf (message, "%s\r", name);
  send(&message);
  waitsend();
 }

/*========================================================
Zobrazeni vlhkosti
========================================================*/
void disp_hum(void)
 {
  d_point = 0;							// zhasnout tecku
  display_buffer[0] = 0x0b;				// zobrazit H
  if (uti_error == 1)					// otestovat spravnost vysledku 
   {
    display_buffer[1] = 0x0a;			// pri chybe zobrazit "H Er"
    display_buffer[2] = 0x0c;
    display_buffer[3] = 0x0d;
    led_lf_ce = 1;						// a nastavit dlouhe blikani
   }
  else
   {									// jinak zobrazit vlhkost
    led_lf_ce = 0;						// zrusit dlouhe blikani
    if (humidity == 100)				// pro zobrazeni 100% 
     {
      display_buffer[1] = 1;
      display_buffer[2] = 0;
      display_buffer[3] = 0;
     }
    else								// pro normalni zobrazeni vlhkosti
     {
      display_buffer[1] = 0x0a;
      display_buffer[3] = humidity % 10;
      display_buffer[2] = humidity / 10;
      if (display_buffer[2] == 0)
       display_buffer[2] = 0x0a;
     }
   }
 }

/*========================================================
Zobrazeni teploty
========================================================*/
void disp_temp(void)
 {
  int a = temperature;			// prechodne celociselna hodnota teploty

  if (smt_error == 1)
   {							// pri chybe zobrazit "Errr"
    d_point = 0;				// a zhasnout des. tecku
    display_buffer[0] = 0x0c;
    display_buffer[1] = 0x0d;
    display_buffer[2] = 0x0d;
    display_buffer[3] = 0x0d;
    led_sf_ce = 1;				// a nastavit kratke blikani
   }
  else
   {							// jinak zobrazit teplotu
    led_sf_ce = 0;				// vypnout kratke blikani
    d_point = 1;				// rozsvitit des. tecku
    if (temp_minus == 1)		// test zaporne teploty
	 {
      display_buffer[0] = 0x0e;
      display_buffer[1] = 0x0a;
      display_buffer[2] = 0;
      display_buffer[3] = 1;
	 }
	else
     {
      display_buffer[2] = a % 10;	// desitky
      a /= 10;
      display_buffer[1] = a % 10; // jednotky
      a /= 10;
      if (a == 0)					// stovky 
       {							// a potlaceni pocatecnich nul
        display_buffer[0] = 0x0a;
	    if (display_buffer[1] == 0)
	     display_buffer[1] = 0x0a;
       }
      display_buffer[3] = temperature_dec;	// zobrazeni desetin
     }
   }
  }

/*========================================================
Smazani displaye
========================================================*/
/*
void disp_clear(void)
 {
  d_point = 0;					// zruseni des. tecky
  display_buffer[0] = 0x0a;		// nastaveni prazdnych
  display_buffer[1] = 0x0a;		// znaku na display
  display_buffer[2] = 0x0a;
  display_buffer[3] = 0x0a;
  DA0 = 1;						// vypnuti multiplexu
  DA1 = 1;
  DA2 = 1;
  DA3 = 1;
//  CATHODE_PORT = led_buffer;	// zobrazit LED
  DALED = 1;
 }
*/

/*##################################################################

						Funkce nastaveni

##################################################################*/

/*========================================================
Kopirovani nastavenych hodnot z EEPROM
========================================================*/
void  load_setings(void)			
 {
  unsigned char a, b, c;

#if AT8252					// kopirovani nastavenych hodnot z EEPROM

  WMCON |= EEMEN_;
  addr = ee_addr;				
  set_temp = ee_set_temp;
  set_t_hyst = ee_set_t_hyst;
  set_hum = ee_set_hum;
  set_h_v = ee_set_h_v;	
  set_t_v = ee_set_t_v;	
  set_lt_w = ee_set_lt_w;
  calib_k_a = ee_calib_k_a;
  calib_k_b = ee_calib_k_b;
  calib_k_ok = ee_calib_k_ok;
  WMCON &= ~(EEMEN_);

  a = calib_k_a >> 8;				// prevod na char
  b = calib_k_a & 0x00ff;			// prevod na char
  c = a + b + calib_k_b + 47;		// checksum kalibracnich konstant

#else

  addr = I2C_RRead_Byte(EE_ADDR);				
  set_temp = I2C_RRead_Byte(EE_SET_TEMP);
  set_t_hyst = I2C_RRead_Byte(EE_SET_T_HYST);
  set_hum = I2C_RRead_Byte(EE_SET_HUM);
  set_h_v = I2C_RRead_Byte(EE_SET_H_V);	
  set_t_v = I2C_RRead_Byte(EE_SET_T_V);	
  set_lt_w = I2C_RRead_Byte(EE_SET_LT_W);	
  a = I2C_RRead_Byte(EE_CALIB_K_A);
  b = I2C_RRead_Byte(EE_CALIB_K_A + 1);
  calib_k_b = I2C_RRead_Byte(EE_CALIB_K_B);
  calib_k_ok = I2C_RRead_Byte(EE_CALIB_K_OK);

  calib_k_a = a + (b * 0xff);			// prevod na int
  c = a + b + calib_k_b + 0x47;			// checksum kalibracnich konstant

#endif

  if ((addr >= '0' && addr <= '9') || (addr >= 'A' && addr <= 'L')
      || (addr >= 'N' && addr <= 'Z'))	// kontrola spravnych hodnot adresy
   {
   }
  else
   addr = 'A';								// pokud nejsou spravne nastavit A

  if (set_temp > 125)						// test spravne hodnoty teploty 
   set_temp = 25;
  if (set_t_hyst == 0 || set_t_hyst > 5)	// test spravne hodnoty teplotni hystereze 
   set_t_hyst = 2;
  if (set_hum < 10 || set_hum > 90)			// test spravne hodnoty vlhkosti 
   set_hum = 45;
  if (set_h_v < 10 || set_h_v > 40)			// test blokovani funkci
   h_v_en = 0;
  else
   h_v_en = 1;
  if (set_t_v == 0 || set_t_v > 99)
   t_v_en = 0;
  else
   t_v_en = 1;
  if (set_lt_w == 0 || set_lt_w > 99)
   lt_w_en = 0;
  else
   lt_w_en = 1;
  if (calib_k_ok != c)
   {
    calib_k_a = 655;
    calib_k_b = 0;
	calib_ok = 0;
   }
  else
	calib_ok = 1;
 }

/*========================================================
Nastaveni teploty
========================================================*/
void set_temperature(void)
{
 unsigned char j;
 char a[4];

  for (j = 0; j < 3; j++)
   a[j] = command[j+4];
  a[3] = '\0';
  j = atoi (a);
  if (j >= 0 && j <= 125)		// kontrola spravne zadanych hodnot
   {
    send_ok();
#if AT8252
     p_eeprom = &ee_set_temp;
     ee_write(j);
#else
     p_eeprom = EE_SET_TEMP;
     ee_write(j);
#endif
     set_temp = j;
   }
  else
   send_error();
}

/*========================================================
Nastaveni teplotni hystereze
========================================================*/
void set_t_hysteresis(void)
{
 unsigned char j;
 j = (command[4] - 48);
 if (j >= 1 && j <= 5)		// kontrola spravne zadanych hodnot
  {
   send_ok();
#if AT8252
   p_eeprom = &ee_set_t_hyst;
   ee_write(j);
#else
   p_eeprom = EE_SET_T_HYST;
   ee_write(j);
#endif
   set_t_hyst = j;
  }
 else
  send_error();
}

/*========================================================
Nastaveni nove adresy
========================================================*/
void set_addres(void)
{
 unsigned char j;

 j = command[4];
 if ((j >= '0' && j <= '9') || (j >= 'A' && j <= 'L') || (j >= 'N' && j <= 'Z'))	// kontrola spravne zadanych hodnot
  {
#if AT8252
   p_eeprom = &ee_addr;
   ee_write(j);
#else
   p_eeprom = EE_ADDR;
   ee_write(j);
#endif
   addr = j;
   send_ok();
  }
 else
  send_error();
}

/*========================================================
Nastaveni vlhkosti
========================================================*/
void set_humidity(void)
{
 unsigned char j;
 char a[3];

 a[0] = command[4];
 a[1] = command[5];
 a[2] = '\0';
 j = atoi (a);
 if (j >= 10 && j <= 90)		// kontrola spravne zadanych hodnot
  {
   send_ok();
#if AT8252
   p_eeprom = &ee_set_hum;
   ee_write(j);
#else
   p_eeprom = EE_SET_HUM;
   ee_write(j);
#endif
   set_hum = j;
  }
 else
  send_error();
}

/*========================================================
Nastaveni zvysene vlhkosti pro ventilaci
========================================================*/
void set_hum_v(void)
{
 unsigned char j;
 char a[3];

 a[0] = command[4];
 a[1] = command[5];
 a[2] = '\0';
 j = atoi (a);
 if (j == 0 || (j >= 10 && j <= 40))		// kontrola spravne zadanych hodnot
  {
   send_ok();
#if AT8252
   p_eeprom = &ee_set_h_v;
   ee_write(j);
#else
   p_eeprom = EE_SET_H_V;
   ee_write(j);
#endif
   set_h_v = j;
   if (j == 0)
    h_v_en = 0;
   else
    h_v_en = 1;
  }
 else
  send_error();
}

/*========================================================
Nastaveni zvysene teploty pro ventilaci
========================================================*/
void set_temp_v(void)
{
 unsigned char j;
 char a[3];

 a[0] = command[4];
 a[1] = command[5];
 a[2] = '\0';
 j = atoi (a);
 if (j == 0 || (j >= 3 && j <= 99))		// kontrola spravne zadanych hodnot
  {
   send_ok();
#if AT8252
   p_eeprom = &ee_set_t_v;
   ee_write(j);
#else
   p_eeprom = EE_SET_T_V;
   ee_write(j);
#endif
   set_t_v = j;
   if (j == 0)
    t_v_en = 0;
   else
    t_v_en = 1;
  }
 else
  send_error();
}

/*========================================================
Nastaveni blokovani zalevani pri nizke teplote
========================================================*/
void set_ltemp_w(void)
{
 unsigned char j;
 char a[3];

 a[0] = command[4];
 a[1] = command[5];
 a[2] = '\0';
 j = atoi (a);
 if (j >= 0 && j <= 99)		// kontrola spravne zadanych hodnot
  {
   send_ok();
#if AT8252
   p_eeprom = &ee_set_lt_w;
   ee_write(j);
#else
   p_eeprom = EE_SET_LT_W;
   ee_write(j);
#endif
   set_lt_w = j;
   if (j == 0)
    lt_w_en = 1;
   else
    lt_w_en = 0;
  }
 else
  send_error();
}

/*========================================================
Nastaveni kalibracni konstanty a
========================================================*/
void calib_a(void)
{
 unsigned int j;
 unsigned char a, b;
 char s[4];

  for (j = 0; j < 3; j++)
   s[j] = command[j+4];
  s[3] = '\0';
  j = atoi (s);
  if (j >= 0 && j <= 999)		// kontrola spravne zadanych hodnot
   {
    send_ok();
	a = j >> 8;					// prevod na char
	b = j & 0x00ff;
    p_eeprom = EE_CALIB_K_A;
    ee_write(a);
//    p_eeprom = EE_CALIB_K_A + 1;
    ee_write(b);
    calib_k_a = j;
    cal_a_ok = 1;
    calib_ok = 0;
   }
  else
   send_error();
}

/*========================================================
Nastaveni kalibracni konstanty b
========================================================*/
void calib_b(void)
{
 unsigned char a, b, c, j;
 char s[4];

  for (j = 0; j < 3; j++)
   s[j] = command[j+4];
  s[3] = '\0';
  j = atoi (s);
  if (j >= 0 && j <= 255)		// kontrola spravne zadanych hodnot
   {
    send_ok();
	a = calib_k_a >> 8;
	b = calib_k_a & 0x00ff;
    c = a + b + j + 47;			// checksum kalibracnich konstant
#if AT8252
     p_eeprom = &ee_calib_k_b;
     ee_write(j);
     p_eeprom = &ee_calib_k_ok;
     ee_write(c);
#else
     p_eeprom = EE_CALIB_K_B;
     ee_write(j);
     p_eeprom = EE_CALIB_K_OK;
     ee_write(c);
#endif
     calib_k_b = j;
	 calib_ok = 1;
   }
  else
   send_error();
}

/*##################################################################

						Funkce rizeni

##################################################################*/

/*========================================================
Logika ovladani vystupu
========================================================*/
void loc_out_control(void)
 {
  register unsigned char sth;

  if ((set_t_hyst == 1) || ((set_t_hyst % 2) == 1))
   {									// test je-li hodnota hystereze licha
    sth = set_temp + (set_t_hyst / 2);	// hysterezi a teplotu secist
    if ((temperature > sth) || (temperature == sth && temperature_dec >= 5))
     									// porovnat teplotu pro vypnuti
     loc_heat_en = 0;					// prestat topit
    else
     {
	  sth = set_temp - (set_t_hyst / 2) - 1; // hysterezi a teplotu odecist
      if ((temperature < sth) || (temperature == sth && temperature_dec <= 5))
       loc_heat_en = 1;					// zatopit
	 }
   }
  else
   {									// pokud je hystereze suda neuvazuji se desetiny
    sth = set_t_hyst / 2;
    if (temperature >= (set_temp + sth))
     loc_heat_en = 0;					// prestat topit
    else
     {
      if (temperature < (set_temp - sth))
       loc_heat_en = 1;					// zatopit
	 }
   }

  if (h_v_en == 1)							// ovladani vetrani pri vyssi vlhkosti
   {
    if (humidity >= (set_hum + set_h_v))
	 loc_v_h_en = 1;						// vetrat
	else
	 {
      if (humidity < ((set_hum + set_h_v)-5)) // pokud vlhkost poklesne o 5% pod
      loc_v_h_en = 0;						// nastavenou hodnotu tak prestat vetrat
	 }
   }
  else
   loc_v_h_en = 0;							// pro jistotu prestat vetrat

  if (t_v_en == 1)							// ovladani vetrani pri vyssi teplote
   {
    if (temperature >= (set_temp + set_t_v))
	 loc_v_t_en = 1;						// vetrat
	else
	 {
      if (temperature < ((set_temp + set_t_v)-1)) // pokud teplota poklesne o 1 pod
       loc_v_t_en = 0;						// nastavenou hodnotu tak prestat vetrat
	 }
   }
  else
   loc_v_h_en = 0;							// pro jistotu prestat vetrat


  if (lt_w_en == 1 && temperature < set_lt_w && smt_error == 0)// pri nizke teplote 
   loc_wat_en = 0;							// jen vypnout zalevani
  else										// pokud je teplota vyssi nebo je
   {										// funkce blokovana
    if (humidity >= (set_hum + 4))			// ovladat zalevani podle vlkosti
     loc_wat_en = 0;						// vypnoput zalevani
    else
     {
      if (humidity < (set_hum - 5))
        loc_wat_en = 1;						// zalevat
     }
   }
  if (smt_error == 1)					// pri chybe cidla teploty
   {
    loc_heat_en = 0;					// zakazat topeni
    loc_v_t_en = 0;						// zakazat vetrani podle teploty
   }
  if (uti_error == 1)					// pri chybe cidla vlhkosti
   {
    loc_wat_en = 0;						// zakazat zalevani
    loc_v_h_en = 0;						// zakazat vetrani podle vlhkosti
   }
 }

/*========================================================
Volba manualniho ovladani vystupu pro topeni
========================================================*/
void s_h_man(void)
 {
  switch(command[4])
   {
    case '0':
	 man_heat = 0;					// vynulovat priznak manualniho ovladani
	 led_f_heat = 0;
	 led_sf_heat = 0;
     com_heat_en = 0;				// vynulovat priznak ovladani
     send_ok();
     break;
    case '1':
	 man_heat = 1;					// nastavit priznak manualniho ovladani
	 led_heat = 1;					// nastavit LEDky
	 led_f_heat = 1;
	 led_sf_heat = 1;
     loc_heat_en = 0;				// vynulovat priznak logickeho ovladani
	 com_heat_en = 0;				// vynulovat priznak ovladani
     send_ok();
     break;
    default:
     send_error();
     break;
   }
 }

/*========================================================
Volba manualniho ovladani vystupu pro vetrani
========================================================*/
void s_v_man(void)
 {
  switch(command[4])
   {
    case '0':
	 man_vent = 0;					// vynulovat priznak manualniho ovladani
	 led_f_vent = 0;
	 led_sf_vent = 0;
     com_vent_en = 0;				// vynulovat priznak ovladani
     send_ok();
     break;
    case '1':
	 man_vent = 1;					// nastavit priznak manualniho ovladani
	 led_vent = 1;					// nastavit LEDky
	 led_f_vent = 1;
	 led_sf_vent = 1;
     loc_v_t_en = 0;				// vynulovat priznaky logickeho ovladani
     loc_v_h_en = 0;
	 com_vent_en = 0;				// vynulovat priznak ovladani
     send_ok();
     break;
    default:
     send_error();
     break;
   }
 }

/*========================================================
Volba manualniho ovladani vystupu pro zalevani
========================================================*/
void s_w_man(void)
 {
  switch(command[4])
   {
    case '0':
	 man_wat = 0;					// vynulovat priznak manualniho ovladani
	 led_f_water = 0;
	 led_sf_water = 0;
     com_wat_en = 0;				// vynulovat priznak ovladani
     send_ok();
     break;
    case '1':
	 man_wat = 1;					// nastavit priznak manualniho ovladani
	 led_water = 1;					// nastavit LEDky
	 led_f_water = 1;
	 led_sf_water = 1;
     loc_wat_en = 0;				// vynulovat priznak logickeho ovladani
	 com_wat_en = 0;				// vynulovat priznak ovladani
     send_ok();
     break;
    default:
     send_error();
     break;
   }
 }

/*========================================================
Ovladani vystupu pro topeni prikazem
========================================================*/
void s_h_cont(void)
 {
  if (man_heat == 1)
   {
    switch(command[4])
     {
      case '0':
	   com_heat_en = 0;				// vynulovat priznak ovladani
	   loc_heat_en = 0;				// i priznak logiky
       led_sf_heat = 1;
       send_ok();
	   break;
	  case '1':
	   com_heat_en = 1;				// nastavit priznak ovladani
       led_sf_heat = 0;
       send_ok();
	   break;
	  default:
       send_error();
      break;
     }
   }
  else
   send_merror();
 }

/*========================================================
Ovladani vystupu pro vetrani prikazem
========================================================*/
void s_v_cont(void)
 {
  if (man_vent == 1)
   {
    switch(command[4])
     {
      case '0':
	   com_vent_en = 0;				// vynulovat priznak ovladani
	   loc_v_t_en = 0;				// i priznaky logiky
	   loc_v_h_en = 0;
       led_sf_vent = 1;
       send_ok();
	   break;
	  case '1':
	   com_vent_en = 1;				// nastavit priznak ovladani
       led_sf_vent = 0;
       send_ok();
	   break;
	  default:
       send_error();
       break;
     }
   }
  else
   send_merror();
 }

/*========================================================
Ovladani vystupu pro zalevani prikazem
========================================================*/
void s_w_cont(void)
 {
  if (man_wat == 1)
   {
    switch(command[4])
     {
      case '0':
	   com_wat_en = 0;				// vynulovat priznak ovladani
	   loc_wat_en = 0;				// i priznak logiky
       led_sf_water = 1;
       send_ok();
	   break;
	  case '1':
	   com_wat_en = 1;				// nastavit priznak ovladani
       led_sf_water = 0;
       send_ok();
	   break;
	  default:
       send_error();
       break;
     }
   }
  else
   send_merror();
 }

/*========================================================
Silove ovladani vystupu
========================================================*/
void out_p_cont(void)
 {
  if ((man_heat != 1 && loc_heat_en == 1) || (man_heat == 1 && com_heat_en == 1))
   {								// testovat priznaky ovladani vystupu
    HEAT = 0;						// zatopit
	if (man_heat != 1) 
     led_heat = 0;
   }
  else
   {
    HEAT = 1;						// prestat topit
	if (man_heat != 1) 
     led_heat = 1;
   }

  if ((man_vent != 1 && (loc_v_t_en == 1 || loc_v_h_en == 1)) || (man_vent == 1 && com_vent_en == 1))
   {								// testovat priznaky ovladani vystupu
    VENT = 0;						// vetrat
    if (man_vent != 1)
     led_vent = 0;
   }
  else
   {
    VENT = 1;						// prestat vetrat
    if (man_vent != 1)
     led_vent = 1;
   }

  if ((man_wat != 1 && loc_wat_en == 1) || (man_wat == 1 && com_wat_en == 1))
   {								// testovat priznaky ovladani vystupu
    WATER = 0;						// zalevat
    led_water = 0;
   }
  else
   {
    WATER = 1;						// prestat zalevat
    led_water = 1;
   }
 }

/*========================================================
Zpracovani comandu
========================================================*/
void proccomm(void)
 {
  switch (command[2])               // test comandu
   {
    case 'R':                       // instrukce cteni
	 switch (command[3])
	  {
	   case 'T':                	// 1. parametr teplota
	    switch (command[4])
		 {
		  case 'A':               	// 2. parametr aktualni 
		   send_temp('A');
		   break;
   		  case 'S':					// 2. parametr nastavena
		   send_temp('S');
		   break;
          default:
           send_error();
	     }
        break;

	   case 'H':                	// 1. parametr vhkost
	    switch (command[4])
		 {
		  case 'A':               	// 2. parametr aktualni 
		   send_hum('A');
		   break;
   		  case 'S':					// 2. parametr nastavena
		   send_hum('S');
		   break;
          default:
           send_error();
	     }
        break;

	   case 'O':                	// 1. parametr vystupy
	    switch (command[4])
		 {
		  case 'H':               	// 2. parametr topeni
		   send_out_s(HEAT, 'H', man_heat);
		   break;
   		  case 'V':					// 2. parametr ventilace
		   send_out_s(VENT, 'V', man_vent);
		   break;
   		  case 'W':					// 2. parametr voda
		   send_out_s(WATER, 'W', man_wat);
		   break;
          default:
           send_error();
	     }
        break;
       default:
        send_error();
	  }
     break;

    case 'S':                       // instrukce nastaveni
	 switch (command[3])
	  {
	   case 'T':                	// 1. parametr teplota
	    set_temperature();
        break;
	   case 'O':                	// 1. parametr hystereze
	    set_t_hysteresis();
        break;
	   case 'H':                	// 1. parametr vlhkost
	    set_humidity();
        break;
	   case 'N':                	// 1. parametr nova adresa
	    set_addres();
        break;
	   case 'A':                	// 1. parametr aktivace posilani hodnot
        switch (command[4])
		 {
		  case 'T':					// 2. parametr teplota
		   s_temp = 1;
  	       send_ok();
		   break;
		  case 'H':					// 2. parametr vlhkost
		   s_hum = 1;
  	       send_ok();
		   break;
          default:
           send_error();
		 }
		break;
	   case 'D':                	// 1. parametr deaktivace posilani hodnot
        switch (command[4])
		 {
		  case 'T':					// 2. parametr teplota
		   s_temp = 0;
  	       send_ok();
		   break;
		  case 'H':					// 2. parametr vlhkost
		   s_hum = 0;
  	       send_ok();
		   break;
          default:
           send_error();
		 }
		break;
       case 'V':                	// 1. parametr vlhkost ventilace
	    set_hum_v();
        break;
	   case 'F':                	// 1. parametr teplota ventilace
	    set_temp_v();
        break;
	   case 'W':                	// 1. parametr teplota zalevani
	    set_ltemp_w();
        break;

       default:
        send_error();
        break;
      }
     break;

    case 'O':                       // instrukce ovladani vystupu
	 switch (command[3])
	  {
	   case 'H':                	// 1. parametr topeni
        s_h_cont();
        break;
	   case 'V':                	// 1. parametr vetrani
        s_v_cont();
        break;
	   case 'W':                	// 1. parametr zalevani
        s_w_cont();
        break;
       default:
        send_error();
        break;
      }
     break;

    case 'M':                       // instrukce manualniho ovladani vystupu
	 switch (command[3])
	  {
	   case 'H':                	// 1. parametr topeni
        s_h_man();
        break;
	   case 'V':                	// 1. parametr vetrani
        s_v_man();
        break;
	   case 'W':                	// 1. parametr zalevani
        s_w_man();
        break;
       default:
        send_error();
        break;
      }
     break;

    case 'C':                       // instrukce pro kalibraci
	 switch (command[3])
	  {
	   case 'A':                	// 1. parametr konstanta a
        calib_a();
        break;
	   case 'B':                	// 1. parametr konstanta a
	    if (cal_a_ok == 1)			// kontrola 
         calib_b();
		else
         send_calaf();
        break;
	   case 'R':                	// 1. parametr cteni konstant
        read_calib();
        break;
       default:
        send_error();
        break;
      }
     break;

    default:
     send_error();
     break;
   }
 }

/*========================================================
Operace provadene jednou za sekundu
=========================================================*/
void new_sec(void)
{
 static unsigned char ns_count = 1; // pocitani cyklu
 unsigned int a;		// prechodne celociselna hodnota teploty

 switch(ns_count)
  {
   case 1:					// prvni sekunda cyklu, mereni vlkosti
//    disp_clear();
	mes_uti();
    a = uti_calc(phase[0], phase[2], phase[1]);	// prepocitat na int 
    humidity = ((a / calib_k_a) - calib_k_b);	// prepocitat na %
    if (s_hum == 1)			// pri nastavenem posilani vysledku na serial
     {
      if (uti_error == 1)		// pri chybe cidla
	   {
        send(&herror);			// poslat chybove hlaseni
	    waitsend();
	   }
	  else						// jinak poslat vysledek
	   send_hum('A');
     }
    disp_hum();
	ns_count++;
	break;
   case 2:					// druha pouze cekani
	ns_count++;
	break;
   case 3:					// treti sekunda, mereni teploty
//    disp_clear();
	mes_smt();
    if (s_temp == 1)		// pri nastavenem posilani vysledku na serial
     {
      if (smt_error == 1)		// pri nepritomnosti cidla
	   {
        send(&terror);
	    waitsend();
	   }
	  else
	   send_temp('A');
     }
    temp_calc();
    disp_temp();
	ns_count++;
	break;
   case 4:					// ctvrta pouze cekani
	ns_count++;
	break;
   case 5:					// pata pouze cekani a
    loc_out_control();		// logika ovladani vystupu
	ns_count = 1;
	break;
   default:
    ns_count = 1;
  }  
 out_p_cont();				// kazdou sekundu provest obsluhu vystupu
}

/*##################################################################

					Funkce inicializace

##################################################################*/

/*========================================================
Inicializace
========================================================*/
void init(void)
 {
  EA = 0;					// zakaz preruseni

#if	AT8252
  WMCON = 0xa3;								// WDT 512mS 
#endif

  T2CON = 0x04;								// timer2 pouzivat jako casovac prodlevy po resetu
  RCAP2H = 0x4c;							// timer2 50mS
  RCAP2L = 0x00;
  TH2 = 0x4c;
  TL2 = 0;
  ET2 = 1;
  res_timer = 1;							// po resetu je vzhledem k bloku ISP nutno cca 500mS pockat
  EA = 1;
  DA0 = 0;
  DA1 = 0;
  DA2 = 0;
  DA3 = 0;
  DALED = 1;
  CATHODE_PORT = 0xfe;

  while (res_timer == 1)					// jako casovac slouzi timer2
   {
    if (wdt_t2_en == 1)
	 {
	  res_wdt();							// pokud vse probiha spravne je kazdych 50mS resetovan WDT
	  wdt_t2_en = 0;
      CATHODE_PORT = CATHODE_PORT >> 1;		// jen pro efekt :-)
	 }
	}
  DA0 = 1;
  DA1 = 1;
  DA2 = 1;
  DA3 = 1;
  ET2 = 0;
  TMOD = 0x21;				// timer0 mod1 16bit
  TH1 = 0xFD;				// nastavit timer1 na 9600Bd
  SCON = 0x50;
  PCON = 0x00;				// standartni rychlost
  TCON = 0x41;				// spustit tim1 a nastavit INT0 na hranu
  T2CON = 0x04;
  TH2 = 0xff;
  TL2 = 0x00;
  RCAP2H = 0xf9;			// nastavit timer 2 na cca 1mS
  RCAP2L = 0x00;
  TL0 = 0;					// vynulovat casovac 0 
  TH0 = 0;
  TR0 = 0;
  ET2 = 1;					// spustit timer 2
  IP = 0x03;				// nastavit priority preruseni int0 a tim0 na 1

  DIR = 0;
  res_wdt();				// pro jistotu reset wdt
  load_setings();			// kopirovani nastavenych hodnot z EEPROM
  mest = 0;					// nulovat priznak mereni casu
  mesdone = 0;				// nulovat priznak uspesneho mereni
  ET0 = 1;
  ES = 1;
  EA = 1;

 }

/*##################################################################

				Funkce obsluhy preruseni

##################################################################*/

/*========================================================
Obsluha preruseni Int0 pro mereni casu
=========================================================*/
void mes_time(void) interrupt 0
 {
//  static
 unsigned int mtimer = 0;		// pro jistotu definovane jako static
  static unsigned char h, l;		// pro jistotu definovane jako static
  static unsigned char mescount;

  h = TH0;				// ulozit stav citace
  l = TL0;				// napred jako byte
  TH0 = 0;				// "vynulovat" casovac 0 POZOR hodnota TL0 MUSI
  TL0 = 2;				// odpovidat poctu instr. mezi ctenim a nul.!!!!!!!
  if (l < 2)			// pokud doslo k preteceni timeru0 behem prepisu hodnot
   h++;					// zvysit h
  if (mest == 0)		// otestovat zacatek mereni
   {
	mest = 1;			// oznacit mereni
	mesfi = 1;			// a mereni prvniho pulzu
	messtart = 0;		// mereni odstartovano, zrusit priznak
	phase[0] = 0;
	return;
   }
  mtimer = h << 8;		// timer ulozeny jako byte
  mtimer += l;			// prepocitat na int
  if (mesfi == 1)
   {
    if ((phase[0] > (mtimer - 10)) && (phase[0] < (mtimer + 10)) && mtimer <= 25000)
     {					// test jsou-li dva male puzly shodne a mensi jak cca 25mS
	  phase[0] += mtimer; // tak je secist
  	  mesfi = 0;		// a oznacit puzl za zmereny
	  mescount = 2;
      return;
	 }
	else
	 phase[0] = mtimer;	// jinak jen aktualizovat phase [0]
	return;
   }
  switch(mescount)
   {
    case 2:
	 phase[1] = mtimer;	// vysledek mereni 2. faze
	 mescount++;
     break;
    case 3:
	 phase[2] = mtimer;	// vysledek mereni 3. faze
         mescount = 0;
	 mesdone = 1;		// oznacit konec mereni
	 mest = 0;
	 EX0 = 0;			// zakazat preruseni
	 ES = 1;			// povolit preruseni serialu
     break;
    default:
     mescount = 0;
   }
 }

/*========================================================
Obsluha preruseni Timer0 pro timeout mereni
=========================================================*/
void timer0(void) interrupt 1
 {
  static bit msttout;						// priznak timeoutu prijmu
  static unsigned char temp_count;			// pomocny citac

  if (messtart == 1)		// test timeotu 1s pri startu mereni
   {
    if (msttout == 0)
	 {
	  msttout = 1;
	  temp_count = 0;
	 }
    temp_count++;
	if (temp_count >= 15)
	 {						// prekrocen timeout
      messtart = 0;			// nulovat priznak
	  msttout = 0;
	  uti_error = 1;
	 }
	return;					// ukoncit preruseni
   }
  else
   msttout = 0;
  if (mest == 1)			// pretekl casovac pri mereni delky pulzu
   {					
    mest = 0;				// nulovat priznak
    mesdone = 0;
	uti_error = 1;
	return;					// ukoncit preruseni
   }
 }

/*============================================================================================
Obsluha preruseni od timer2 - pouziti jako casovac pro Start,
                              multiplex displaye a TimeOut prijmu
=============================================================================================*/

void timer2 (void) interrupt 5
 {
  static unsigned char idata count = 0; //, qcount = 0;
  static unsigned char display_count = 0;	// pozice displaye
  static unsigned int time_count = 0;		// citac casu
  static unsigned int idata rec_count;		// pomocny citac timeoutu prijmu
  register unsigned char temp;				// pomocne

  if (res_timer == 1) 
   {
    count++;
    if (res_timer == 1)
     {
      if (count >= 16)
       res_timer = 0;
     }
   }
  else
   {
    time_count++;
    if (time_count == 500)
     {
      time_count = 0;
      n_sec = 1;
     } 
    switch (display_count)
     {
      case 0:												//display 1
       DA0 = 1;
       DA1 = 1;
       DA2 = 1;
       DA3 = 1;
       DALED = 1;
       temp = ~char_tab[(display_buffer[display_count])];
       temp |= 0x80 ;
       CATHODE_PORT = temp;
       DA0 = 0;
       display_count++;  					
       break;
      case 1:												// display 2
       DA0 = 1;
       temp = ~char_tab[(display_buffer[display_count])];
       temp |= 0x80;
       CATHODE_PORT = temp;
       DA1 = 0;
       display_count++;
       break;
      case 2:												// display 2
       DA1 = 1;
       temp = ~char_tab[(display_buffer[display_count])];
	   (d_point == 1) ? (temp &= 0x7f) : (temp |= 0x80);
       CATHODE_PORT = temp;
       DA2 = 0;
       display_count++;
       break;
      case 3:												// display 2
       DA2 = 1;
       temp = ~char_tab[(display_buffer[display_count])];
	   temp |= 0x80;
       CATHODE_PORT = temp;
       DA3 = 0;
       display_count++;
       break;
      case 4:												// ledky
       DA3 = 1;
	   led_buffer &= 0x0f;							// zalohovat nastaveni LED
       temp = led_buffer;
	   temp <<= 4;
	   led_buffer |= temp;

       if (led_lf_ce == 1)							// test dlouheho bliknuti led C/E
        (time_count < 200) ? (led_ce = 0) : (led_ce = 1); //svitit prvnich 400mS
       if (led_sf_ce == 1)							// pro kratky blik svitit od 750 - 900mS
	    {
		 if (time_count > 350 && time_count < 450)
	      led_ce = 0;
	     else
		  led_ce = 1;
		}
       if (led_f_heat == 1)							// test blikani led topeni
	    {
		 if (led_sf_heat == 1)						// pro kratky blik svitit jen 200mS
		  (time_count < 100) ? (led_heat = 0) : (led_heat = 1);
		 else										// pro dlouhy blik svitit 800mS
		  (time_count < 400) ? (led_heat = 0) : (led_heat = 1);
	    }

       if (led_f_vent == 1)							// test blikani led vetrani
	    {
		 if (led_sf_vent == 1)						// pro kratky blik svitit jen 200mS
		  (time_count < 100) ? (led_vent = 0) : (led_vent = 1);
		 else										// pro dlouhy blik svitit 800mS
		  (time_count < 400) ? (led_vent = 0) : (led_vent = 1);
	    }

       if (led_f_water == 1)						// test blikani led zalevani
	    {
		 if (led_sf_water == 1)						// pro kratky blik svitit jen 200mS
		  (time_count < 100) ? (led_water = 0) : (led_water = 1);
		 else										// pro dlouhy blik svitit 800mS
		  (time_count < 400) ? (led_water = 0) : (led_water = 1);
	    }

       temp = led_buffer & 0x0f;
       CATHODE_PORT = temp;							// zobrazit LED
       DALED = 0;
	   led_buffer >>= 4;							// obnovit nastaveni LED
       led_buffer &= 0x0f;
       display_count = 0;
       break;
      default:
       display_count = 0;
     }
   }

  if (receive == 1)			// test timeoutu 5s prijmu
   {
    if (rectout == 0)
	 {
	  rectout = 1;
	  rec_count = 0;
	 }
	else
	 {
      rec_count++;
	  if (rec_count >= 2500)
	   {						// prekrocen timeout
        receive = 0;			// nulovat priznak prijmu
	    rectout = 0;		
	    led_ce = 1;
	    eor = 1;			
	   }
	 }
   }
  else
   rectout = 0;
  wdt_t2_en = 1;
  TF2 = 0;
 }

/*========================================================
Obsluha preruseni Serial
=========================================================*/
void serial (void) interrupt 4
 {
  static unsigned char scount;

  EA = 0;										// pro jistotu

  if (RI == 1)									// prijem
   {
    if (SBUF == '*')							// prijat prefix
     {
      command[0] = '*';
      receive_p = 1;							// oznaceni prefixu
      receive = 0;
//      scount = 1;								// zacatek komunikace
     }
	else
	 {
      if (receive == 1)							// prijem dalsich casti commandu
       {
	    rectout = 0;							// nulovani timeoutu
        command[scount] = SBUF;
        scount++;
        if (scount == 13 || SBUF == 0x0d)		// test konce prijmu
         {
          command[scount] = "\0";
          receive = 0;
          received = 1;
          led_ce = 1;
         }
       }
      else
       {
	    if (receive_p == 1)
	     {
          if (SBUF == addr) 					// prijata adresa
           {
            command[1] = SBUF;
            receive_p = 0;  					// zruseni prefixu
		    receive = 1;						// oznaceni prijmu
            scount = 2;
            led_ce = 0;
           }
		  else
		   receive_p = 0;						// jinak zrusit priznak prefixu

          if (SBUF == '?') 						// prijat dotaz na adresu a jmeno
           {
		    send_n = 1;
            receive_p = 0;  					// zruseni prefixu
		    receive = 0;						// a prijmu
            scount = 0;
		   }
	     }
	   }
	 }
    RI = 0;
   }
  else											// vysilani
   {
    TI = 0;
    if (s_count != s_nob)
     {
      SBUF = *p_s_out;
	  p_s_out++;
	  s_count++;
	 }
	else										// odeslano
	 {
	  s_count = 0;
	  s_nob = 0;
	  transmit = 0;
	  DIR = 0;									// vypnout vysilani v rezimu 485
	  led_ce = 1; 								// zhasnout LED
	 }
   }
  EA = 1;
 }

/*##################################################################

						Funkce Main

##################################################################*/
void main(void)
 {

  init();

/*       sprintf (message, "*M%c ", addr);
       send(&message);
       waitsend();
       sprintf (message, "%s\r", name);
       send(&message);
       waitsend();
*/

  while(1)
   {
    if (n_sec == 1)				// nova sekunda
     {
      new_sec();
      n_sec = 0;
     }
    
	if (received == 1)			// prijata data
     {
      proccomm();
      received = 0;
     }
	if (eor == 1)				// posilani hlaseni o timeoutu serialu
	 {
	  send_quickly();
	  eor = 0;
	 }
	if (send_n == 1)			// posilani jmena
	 {
	  send_name();
	  send_n = 0;
	 }
	if (wdt_t2_en == 1)			// obsluha WDT
	 {
	  wdt_t2_en = 0;
	  res_wdt();
	 }
   }
 }

